#-------------------------------------------------------------------------------
# elftools: dwarf/structs.py
#
# Encapsulation of Construct structs for parsing DWARF, adjusted for correct
# endianness and word-size.
#
# Eli Bendersky (eliben@gmail.com)
# This code is in the public domain
#-------------------------------------------------------------------------------
from logging.config import valid_ident
from ..construct import (
    UBInt8, UBInt16, UBInt32, UBInt64, ULInt8, ULInt16, ULInt32, ULInt64,
    SBInt8, SBInt16, SBInt32, SBInt64, SLInt8, SLInt16, SLInt32, SLInt64,
    Adapter, Struct, ConstructError, If, Enum, Array, PrefixedArray,
    CString, Embed, StaticField, IfThenElse, Construct, Rename, Sequence,
    String, Switch, Value
    )
from ..common.construct_utils import (RepeatUntilExcluding, ULEB128, SLEB128,
    StreamOffset, ULInt24, UBInt24)
from .enums import *


class DWARFStructs(object):
    """ Exposes Construct structs suitable for parsing information from DWARF
        sections. Each compile unit in DWARF info can have its own structs
        object. Keep in mind that these structs have to be given a name (by
        calling them with a name) before being used for parsing (like other
        Construct structs). Those that should be used without a name are marked
        by (+).

        Accessible attributes (mostly as described in chapter 7 of the DWARF
        spec v3):

            Dwarf_[u]int{8,16,32,64):
                Data chunks of the common sizes

            Dwarf_offset:
                32-bit or 64-bit word, depending on dwarf_format

            Dwarf_length:
                32-bit or 64-bit word, depending on dwarf_format

            Dwarf_target_addr:
                32-bit or 64-bit word, depending on address size

            Dwarf_initial_length:
                "Initial length field" encoding
                section 7.4

            Dwarf_{u,s}leb128:
                ULEB128 and SLEB128 variable-length encoding

            Dwarf_CU_header (+):
                Compilation unit header

            Dwarf_TU_header (+):
                Type unit header

            Dwarf_abbrev_declaration (+):
                Abbreviation table declaration - doesn't include the initial
                code, only the contents.

            Dwarf_dw_form (+):
                A dictionary mapping 'DW_FORM_*' keys into construct Structs
                that parse such forms. These Structs have already been given
                dummy names.

            Dwarf_lineprog_header (+):
                Line program header

            Dwarf_lineprog_file_entry (+):
                A single file entry in a line program header or instruction

            Dwarf_CIE_header (+):
                A call-frame CIE

            Dwarf_FDE_header (+):
                A call-frame FDE

        See also the documentation of public methods.
    """

    # Cache for structs instances based on creation parameters. Structs
    # initialization is expensive and we don't won't to repeat it
    # unnecessarily.
    _structs_cache = {}

    def __new__(cls, little_endian, dwarf_format, address_size, dwarf_version=2):
        """ dwarf_version:
                Numeric DWARF version

            little_endian:
                True if the file is little endian, False if big

            dwarf_format:
                DWARF Format: 32 or 64-bit (see spec section 7.4)

            address_size:
                Target machine address size, in bytes (4 or 8). (See spec
                section 7.5.1)
        """
        key = (little_endian, dwarf_format, address_size, dwarf_version)

        if key in cls._structs_cache:
            return cls._structs_cache[key]

        self = super().__new__(cls)
        assert dwarf_format == 32 or dwarf_format == 64
        assert address_size == 8 or address_size == 4, str(address_size)
        self.little_endian = little_endian
        self.dwarf_format = dwarf_format
        self.address_size = address_size
        self.dwarf_version = dwarf_version
        self._create_structs()
        cls._structs_cache[key] = self
        return self

    def initial_length_field_size(self):
        """ Size of an initial length field.
        """
        return 4 if self.dwarf_format == 32 else 12

    def _create_structs(self):
        if self.little_endian:
            self.Dwarf_uint8 = ULInt8
            self.Dwarf_uint16 = ULInt16
            self.Dwarf_uint24 = ULInt24
            self.Dwarf_uint32 = ULInt32
            self.Dwarf_uint64 = ULInt64
            self.Dwarf_offset = ULInt32 if self.dwarf_format == 32 else ULInt64
            self.Dwarf_length = ULInt32 if self.dwarf_format == 32 else ULInt64
            self.Dwarf_target_addr = (
                ULInt32 if self.address_size == 4 else ULInt64)
            self.Dwarf_int8 = SLInt8
            self.Dwarf_int16 = SLInt16
            self.Dwarf_int32 = SLInt32
            self.Dwarf_int64 = SLInt64
        else:
            self.Dwarf_uint8 = UBInt8
            self.Dwarf_uint16 = UBInt16
            self.Dwarf_uint24 = UBInt24
            self.Dwarf_uint32 = UBInt32
            self.Dwarf_uint64 = UBInt64
            self.Dwarf_offset = UBInt32 if self.dwarf_format == 32 else UBInt64
            self.Dwarf_length = UBInt32 if self.dwarf_format == 32 else UBInt64
            self.Dwarf_target_addr = (
                UBInt32 if self.address_size == 4 else UBInt64)
            self.Dwarf_int8 = SBInt8
            self.Dwarf_int16 = SBInt16
            self.Dwarf_int32 = SBInt32
            self.Dwarf_int64 = SBInt64

        # Only instantiate those parsers that are used standalone,
        # as opposed to dispatch tables (e. g. forms, opcodes).
        # In dispatch tables, they are instantiated already.
        # LEB128 parsers are instantiated too, elsewhere.
        self.the_Dwarf_offset = self.Dwarf_offset('')
        self.the_Dwarf_target_addr = self.Dwarf_target_addr('')
        self.the_Dwarf_uint32 = self.Dwarf_uint32('')
        self.the_Dwarf_uint16 = self.Dwarf_uint16('')
        self.the_Dwarf_uint8 = self.Dwarf_uint8('')

        self._create_initial_length()
        self._create_leb128()
        self._create_cu_header()
        self._create_tu_header()
        self._create_abbrev_declaration()
        self._create_dw_form()
        self._create_lineprog_header()
        self._create_callframe_entry_headers()
        self._create_aranges_header()
        self._create_nameLUT_header()
        self._create_string_offsets_table_header()
        self._create_address_table_header()
        self._create_loclists_parsers()
        self._create_rnglists_parsers()

        self._create_debugsup()
        self._create_gnu_debugaltlink()

    def _create_initial_length(self):
        def _InitialLength(name):
            # Adapts a Struct that parses forward a full initial length field.
            # Only if the first word is the continuation value, the second
            # word is parsed from the stream.
            return _InitialLengthAdapter(
                Struct(name,
                    self.Dwarf_uint32('first'),
                    If(lambda ctx: ctx.first == 0xFFFFFFFF,
                        self.Dwarf_uint64('second'),
                        elsevalue=None)))
        self.Dwarf_initial_length = _InitialLength

    def _create_leb128(self):
        self.Dwarf_uleb128 = ULEB128
        self.Dwarf_sleb128 = SLEB128
        self.the_Dwarf_uleb128 = self.Dwarf_uleb128('')
        self.the_Dwarf_sleb128 = self.Dwarf_sleb128('')

    def _create_cu_header(self):
        dwarfv4_CU_header = Struct('',
            self.Dwarf_offset('debug_abbrev_offset'),
            self.Dwarf_uint8('address_size')
        )
        # DWARFv5 reverses the order of address_size and debug_abbrev_offset.
        # DWARFv5 7.5.1.1
        dwarfv5_CP_CU_header = Struct('',                  
            self.Dwarf_uint8('address_size'),
            self.Dwarf_offset('debug_abbrev_offset')
        )
        # DWARFv5 7.5.1.2
        dwarfv5_SS_CU_header = Struct('',
            self.Dwarf_uint8('address_size'),
            self.Dwarf_offset('debug_abbrev_offset'),
            self.Dwarf_uint64('dwo_id')
        )
        # DWARFv5 7.5.1.3
        dwarfv5_TS_CU_header = Struct('',
            self.Dwarf_uint8('address_size'),
            self.Dwarf_offset('debug_abbrev_offset'),
            self.Dwarf_uint64('type_signature'),
            self.Dwarf_offset('type_offset')
        )
        dwarfv5_CU_header = Struct('',
            Enum(self.Dwarf_uint8('unit_type'), **ENUM_DW_UT),
            Embed(Switch('', lambda ctx: ctx.unit_type,
            {
                'DW_UT_compile'       : dwarfv5_CP_CU_header,
                'DW_UT_partial'       : dwarfv5_CP_CU_header,
                'DW_UT_skeleton'      : dwarfv5_SS_CU_header,
                'DW_UT_split_compile' : dwarfv5_SS_CU_header,
                'DW_UT_type'          : dwarfv5_TS_CU_header,
                'DW_UT_split_type'    : dwarfv5_TS_CU_header,
            })))
        self.Dwarf_CU_header = Struct('Dwarf_CU_header',
            self.Dwarf_initial_length('unit_length'),
            self.Dwarf_uint16('version'),
            IfThenElse('', lambda ctx: ctx['version'] >= 5,
                Embed(dwarfv5_CU_header),
                Embed(dwarfv4_CU_header),
            ))

    def _create_tu_header(self):
        self.Dwarf_TU_header = Struct('Dwarf_TU_header',
                                      self.Dwarf_initial_length('unit_length'),
                                      self.Dwarf_uint16('version'),
                                      self.Dwarf_offset('debug_abbrev_offset'),
                                      self.Dwarf_uint8('address_size'),
                                      self.Dwarf_uint64('signature'),
                                      self.Dwarf_offset('type_offset'))

    def _create_abbrev_declaration(self):
        self.Dwarf_abbrev_declaration = Struct('Dwarf_abbrev_entry',
            Enum(self.Dwarf_uleb128('tag'), **ENUM_DW_TAG),
            Enum(self.Dwarf_uint8('children_flag'), **ENUM_DW_CHILDREN),
            RepeatUntilExcluding(
                lambda obj, ctx:
                    obj.name == 'DW_AT_null' and obj.form == 'DW_FORM_null',
                Struct('attr_spec',
                    Enum(self.Dwarf_uleb128('name'), **ENUM_DW_AT),
                    Enum(self.Dwarf_uleb128('form'), **ENUM_DW_FORM),
                    If(lambda ctx: ctx['form'] == 'DW_FORM_implicit_const',
                        self.Dwarf_sleb128('value')))))

    def _create_debugsup(self):
        # We don't care about checksums, for now.
        self.Dwarf_debugsup = Struct('Elf_debugsup',
            self.Dwarf_int16('version'),
            self.Dwarf_uint8('is_supplementary'),
            CString('sup_filename'))

    def _create_gnu_debugaltlink(self):
        self.Dwarf_debugaltlink = Struct('Elf_debugaltlink',
            CString("sup_filename"),
            String("sup_checksum", length=20))

    def _create_dw_form(self):
        self.Dwarf_dw_form = dict(
            DW_FORM_addr=self.the_Dwarf_target_addr,
            DW_FORM_addrx=self.the_Dwarf_uleb128,
            DW_FORM_addrx1=self.the_Dwarf_uint8,
            DW_FORM_addrx2=self.the_Dwarf_uint16,
            DW_FORM_addrx3=self.Dwarf_uint24(''),
            DW_FORM_addrx4=self.the_Dwarf_uint32,

            DW_FORM_block1=self._make_block_struct(self.Dwarf_uint8),
            DW_FORM_block2=self._make_block_struct(self.Dwarf_uint16),
            DW_FORM_block4=self._make_block_struct(self.Dwarf_uint32),
            DW_FORM_block=self._make_block_struct(self.Dwarf_uleb128),

            # All DW_FORM_data<n> forms are assumed to be unsigned
            DW_FORM_data1=self.the_Dwarf_uint8,
            DW_FORM_data2=self.the_Dwarf_uint16,
            DW_FORM_data4=self.the_Dwarf_uint32,
            DW_FORM_data8=self.Dwarf_uint64(''),
            DW_FORM_data16=Array(16, self.the_Dwarf_uint8), # Used for hashes and such, not for integers
            DW_FORM_sdata=self.the_Dwarf_sleb128,
            DW_FORM_udata=self.the_Dwarf_uleb128,

            DW_FORM_string=CString(''),
            DW_FORM_strp=self.the_Dwarf_offset,
            DW_FORM_strp_sup=self.the_Dwarf_offset,
            DW_FORM_line_strp=self.the_Dwarf_offset,
            DW_FORM_strx1=self.the_Dwarf_uint8,
            DW_FORM_strx2=self.the_Dwarf_uint16,
            DW_FORM_strx3=self.Dwarf_uint24(''),
            DW_FORM_strx4=self.Dwarf_uint64(''),
            DW_FORM_flag=self.the_Dwarf_uint8,

            DW_FORM_ref=self.the_Dwarf_uint32,
            DW_FORM_ref1=self.the_Dwarf_uint8,
            DW_FORM_ref2=self.the_Dwarf_uint16,
            DW_FORM_ref4=self.the_Dwarf_uint32,
            DW_FORM_ref_sup4=self.the_Dwarf_uint32,
            DW_FORM_ref8=self.Dwarf_uint64(''),
            DW_FORM_ref_sup8=self.Dwarf_uint64(''),
            DW_FORM_ref_udata=self.the_Dwarf_uleb128,
            DW_FORM_ref_addr=self.the_Dwarf_target_addr if self.dwarf_version == 2 else self.the_Dwarf_offset,

            DW_FORM_indirect=self.the_Dwarf_uleb128,
            
            # Treated separatedly while parsing, but here so that all forms resovle
            DW_FORM_implicit_const=None,

            # New forms in DWARFv4
            DW_FORM_flag_present = StaticField('', 0),
            DW_FORM_sec_offset = self.the_Dwarf_offset,
            DW_FORM_exprloc = self._make_block_struct(self.Dwarf_uleb128),
            DW_FORM_ref_sig8 = self.Dwarf_uint64(''),

            DW_FORM_GNU_strp_alt=self.the_Dwarf_offset,
            DW_FORM_GNU_ref_alt=self.the_Dwarf_offset,
            DW_AT_GNU_all_call_sites=self.the_Dwarf_uleb128,

            # New forms in DWARFv5
            DW_FORM_loclistx=self.the_Dwarf_uleb128,
            DW_FORM_rnglistx=self.the_Dwarf_uleb128
        )

    def _create_aranges_header(self):
        self.Dwarf_aranges_header = Struct("Dwarf_aranges_header",
            self.Dwarf_initial_length('unit_length'),
            self.Dwarf_uint16('version'),
            self.Dwarf_offset('debug_info_offset'), # a little tbd
            self.Dwarf_uint8('address_size'),
            self.Dwarf_uint8('segment_size')
            )

    def _create_nameLUT_header(self):
        self.Dwarf_nameLUT_header = Struct("Dwarf_nameLUT_header",
            self.Dwarf_initial_length('unit_length'),
            self.Dwarf_uint16('version'),
            self.Dwarf_offset('debug_info_offset'),
            self.Dwarf_length('debug_info_length')
            )

    def _create_string_offsets_table_header(self):
        self.Dwarf_string_offsets_table_header = Struct(
            "Dwarf_string_offets_table_header",
            self.Dwarf_initial_length('unit_length'),
            self.Dwarf_uint16('version'),
            self.Dwarf_uint16('padding'),
            )

    def _create_address_table_header(self):
        self.Dwarf_address_table_header = Struct("Dwarf_address_table_header",
            self.Dwarf_initial_length('unit_length'),
            self.Dwarf_uint16('version'),
            self.Dwarf_uint8('address_size'),
            self.Dwarf_uint8('segment_selector_size'),
            )

    def _create_lineprog_header(self):
        # A file entry is terminated by a NULL byte, so we don't want to parse
        # past it. Therefore an If is used.
        self.Dwarf_lineprog_file_entry = Struct('file_entry',
            CString('name'),
            If(lambda ctx: len(ctx.name) != 0,
                Embed(Struct('',
                    self.Dwarf_uleb128('dir_index'),
                    self.Dwarf_uleb128('mtime'),
                    self.Dwarf_uleb128('length')))))

        class FormattedEntry(Construct):
            # Generates a parser based on a previously parsed piece,
            # similar to deprecared Dynamic.
            # Strings are resolved later, since it potentially requires
            # looking at another section.
            def __init__(self, name, structs, format_field):
                Construct.__init__(self, name)
                self.structs = structs
                self.format_field = format_field

            def _parse(self, stream, context):
                # Somewhat tricky technique here, explicitly writing back to the context
                if self.format_field + "_parser" in context:
                    parser = context[self.format_field + "_parser"]
                else:
                    fields = tuple(
                        Rename(f.content_type, self.structs.Dwarf_dw_form[f.form])
                        for f in context[self.format_field])
                    parser = Struct('formatted_entry', *fields)
                    context[self.format_field + "_parser"] = parser
                return parser._parse(stream, context)

        ver5 = lambda ctx: ctx.version >= 5

        self.Dwarf_lineprog_header = Struct('Dwarf_lineprog_header',
            self.Dwarf_initial_length('unit_length'),
            self.Dwarf_uint16('version'),
            If(ver5,
                self.Dwarf_uint8("address_size"),
                None),
            If(ver5,
                self.Dwarf_uint8("segment_selector_size"),
                None),
            self.Dwarf_offset('header_length'),
            self.Dwarf_uint8('minimum_instruction_length'),
            If(lambda ctx: ctx.version >= 4,
                self.Dwarf_uint8("maximum_operations_per_instruction"),
                1),
            self.Dwarf_uint8('default_is_stmt'),
            self.Dwarf_int8('line_base'),
            self.Dwarf_uint8('line_range'),
            self.Dwarf_uint8('opcode_base'),
            Array(lambda ctx: ctx.opcode_base - 1,
                  self.Dwarf_uint8('standard_opcode_lengths')),
            If(ver5,
                PrefixedArray(
                    Struct('directory_entry_format',
                        Enum(self.Dwarf_uleb128('content_type'), **ENUM_DW_LNCT),
                        Enum(self.Dwarf_uleb128('form'), **ENUM_DW_FORM)),
                    self.Dwarf_uint8("directory_entry_format_count"))),
            If(ver5, # Name deliberately doesn't match the legacy object, since the format can't be made compatible
                PrefixedArray(
                    FormattedEntry('directories', self, "directory_entry_format"),
                    self.Dwarf_uleb128('directories_count'))),
            If(ver5,
                PrefixedArray(
                    Struct('file_name_entry_format',
                        Enum(self.Dwarf_uleb128('content_type'), **ENUM_DW_LNCT),
                        Enum(self.Dwarf_uleb128('form'), **ENUM_DW_FORM)),
                    self.Dwarf_uint8("file_name_entry_format_count"))),
            If(ver5,
                PrefixedArray(
                    FormattedEntry('file_names', self, "file_name_entry_format"),
                    self.Dwarf_uleb128('file_names_count'))),
            # Legacy  directories/files - DWARF < 5 only
            If(lambda ctx: ctx.version < 5,
                RepeatUntilExcluding(
                    lambda obj, ctx: obj == b'',
                    CString('include_directory'))),
            If(lambda ctx: ctx.version < 5,
                RepeatUntilExcluding(
                    lambda obj, ctx: len(obj.name) == 0,
                    self.Dwarf_lineprog_file_entry)) # array name is file_entry
        )

    def _create_callframe_entry_headers(self):
        self.Dwarf_CIE_header = Struct('Dwarf_CIE_header',
            self.Dwarf_initial_length('length'),
            self.Dwarf_offset('CIE_id'),
            self.Dwarf_uint8('version'),
            CString('augmentation'),
            If(lambda ctx: ctx.version >= 4, self.Dwarf_uint8('address_size')),
            If(lambda ctx: ctx.version >= 4, self.Dwarf_uint8('segment_size')),
            self.Dwarf_uleb128('code_alignment_factor'),
            self.Dwarf_sleb128('data_alignment_factor'),
            IfThenElse('return_address_register', lambda ctx: ctx.version > 1,
                self.Dwarf_uleb128(''),
                self.Dwarf_uint8('')))
        self.EH_CIE_header = self.Dwarf_CIE_header

        # The CIE header was modified in DWARFv4, but the
        # CIE header version is driven by the version # in the header
        # itself, independent of the DWARF version
        # in the CUs.

        self.Dwarf_FDE_header = Struct('Dwarf_FDE_header',
            self.Dwarf_initial_length('length'),
            self.Dwarf_offset('CIE_pointer'),
            self.Dwarf_target_addr('initial_location'),
            self.Dwarf_target_addr('address_range'))

    def _make_block_struct(self, length_field):
        """ Create a struct for DW_FORM_block<size>
        """
        return PrefixedArray(
                    subcon=self.Dwarf_uint8('elem'),
                    length_field=length_field(''))

    def _create_loclists_parsers(self):
        """ Create a struct for debug_loclists CU header, DWARFv5, 7,29
        """
        self.Dwarf_loclists_CU_header = Struct('Dwarf_loclists_CU_header',
            StreamOffset('cu_offset'),
            self.Dwarf_initial_length('unit_length'),
            Value('is64', lambda ctx: ctx.is64),
            StreamOffset('offset_after_length'),
            self.Dwarf_uint16('version'),
            self.Dwarf_uint8('address_size'),
            self.Dwarf_uint8('segment_selector_size'),
            self.Dwarf_uint32('offset_count'),
            StreamOffset('offset_table_offset'))

        cld = self.Dwarf_loclists_counted_location_description = PrefixedArray(self.Dwarf_uint8('loc_expr'), self.the_Dwarf_uleb128)

        self.Dwarf_loclists_entries = RepeatUntilExcluding(
            lambda obj, ctx: obj.entry_type == 'DW_LLE_end_of_list',
            Struct('entry',
                StreamOffset('entry_offset'),
                Enum(self.Dwarf_uint8('entry_type'), **ENUM_DW_LLE),
                Embed(Switch('', lambda ctx: ctx.entry_type,
                {
                    'DW_LLE_end_of_list'      : Struct('end_of_list'),
                    'DW_LLE_base_addressx'    : Struct('base_addressx', self.Dwarf_uleb128('index')),
                    'DW_LLE_startx_endx'      : Struct('startx_endx', self.Dwarf_uleb128('start_index'), self.Dwarf_uleb128('end_index'), cld),
                    'DW_LLE_startx_length'    : Struct('startx_endx', self.Dwarf_uleb128('start_index'), self.Dwarf_uleb128('length'), cld),
                    'DW_LLE_offset_pair'      : Struct('startx_endx', self.Dwarf_uleb128('start_offset'), self.Dwarf_uleb128('end_offset'), cld),
                    'DW_LLE_default_location' : Struct('default_location', cld),
                    'DW_LLE_base_address'     : Struct('base_address', self.Dwarf_target_addr('address')),
                    'DW_LLE_start_end'        : Struct('start_end', self.Dwarf_target_addr('start_address'), self.Dwarf_target_addr('end_address'), cld),
                    'DW_LLE_start_length'     : Struct('start_length', self.Dwarf_target_addr('start_address'), self.Dwarf_uleb128('length'), cld),
                })),
                StreamOffset('entry_end_offset'),
                Value('entry_length', lambda ctx: ctx.entry_end_offset - ctx.entry_offset)))

        self.Dwarf_locview_pair = Struct('locview_pair',
            StreamOffset('entry_offset'), self.Dwarf_uleb128('begin'), self.Dwarf_uleb128('end'))

    def _create_rnglists_parsers(self):
        self.Dwarf_rnglists_CU_header = Struct('Dwarf_rnglists_CU_header',
            StreamOffset('cu_offset'),
            self.Dwarf_initial_length('unit_length'),
            Value('is64', lambda ctx: ctx.is64),
            StreamOffset('offset_after_length'),
            self.Dwarf_uint16('version'),
            self.Dwarf_uint8('address_size'),
            self.Dwarf_uint8('segment_selector_size'),
            self.Dwarf_uint32('offset_count'),
            StreamOffset('offset_table_offset'))

        self.Dwarf_rnglists_entries = RepeatUntilExcluding(
            lambda obj, ctx: obj.entry_type == 'DW_RLE_end_of_list',
            Struct('entry',
                StreamOffset('entry_offset'),
                Enum(self.Dwarf_uint8('entry_type'), **ENUM_DW_RLE),
                Embed(Switch('', lambda ctx: ctx.entry_type,
                {
                    'DW_RLE_end_of_list'      : Struct('end_of_list'),
                    'DW_RLE_base_addressx'    : Struct('base_addressx', self.Dwarf_uleb128('index')),
                    'DW_RLE_startx_endx'      : Struct('startx_endx', self.Dwarf_uleb128('start_index'), self.Dwarf_uleb128('end_index')),
                    'DW_RLE_startx_length'    : Struct('startx_endx', self.Dwarf_uleb128('start_index'), self.Dwarf_uleb128('length')),
                    'DW_RLE_offset_pair'      : Struct('startx_endx', self.Dwarf_uleb128('start_offset'), self.Dwarf_uleb128('end_offset')),
                    'DW_RLE_base_address'     : Struct('base_address', self.Dwarf_target_addr('address')),
                    'DW_RLE_start_end'        : Struct('start_end', self.Dwarf_target_addr('start_address'), self.Dwarf_target_addr('end_address')),
                    'DW_RLE_start_length'     : Struct('start_length', self.Dwarf_target_addr('start_address'), self.Dwarf_uleb128('length'))
                })),
                StreamOffset('entry_end_offset'),
                Value('entry_length', lambda ctx: ctx.entry_end_offset - ctx.entry_offset)))


class _InitialLengthAdapter(Adapter):
    """ A standard Construct adapter that expects a sub-construct
        as a struct with one or two values (first, second).
    """
    def _decode(self, obj, context):
        if obj.first < 0xFFFFFF00:
            context['is64'] = False
            return obj.first
        else:
            if obj.first == 0xFFFFFFFF:
                context['is64'] = True
                return obj.second
            else:
                raise ConstructError("Failed decoding initial length for %X" % (
                    obj.first))



#-------------------------------------------------------------------------------
# elftools tests
#
# Eli Bendersky (eliben@gmail.com)
# This code is in the public domain
#-------------------------------------------------------------------------------
import unittest
from io import BytesIO

from elftools.dwarf.callframe import (
    CallFrameInfo, CIE, FDE, instruction_name, CallFrameInstruction,
    RegisterRule, DecodedCallFrameTable, CFARule)
from elftools.dwarf.structs import DWARFStructs
from elftools.dwarf.descriptions import (describe_CFI_instructions,
    set_global_machine_arch)
from elftools.dwarf.enums import DW_EH_encoding_flags
from elftools.elf.elffile import ELFFile
from os.path import join


class TestCallFrame(unittest.TestCase):
    def assertInstruction(self, instr, name, args):
        self.assertIsInstance(instr, CallFrameInstruction)
        self.assertEqual(instruction_name(instr.opcode), name)
        self.assertEqual(instr.args, args)

    def test_spec_sample_d6(self):
        # D.6 sample in DWARFv3
        s = BytesIO()
        data = (b'' +
            # first comes the CIE
            b'\x20\x00\x00\x00' +        # length
            b'\xff\xff\xff\xff' +        # CIE_id
            b'\x03\x00\x04\x7c' +        # version, augmentation, caf, daf
            b'\x08' +                    # return address
            b'\x0c\x07\x00' +
            b'\x08\x00' +
            b'\x07\x01' +
            b'\x07\x02' +
            b'\x07\x03' +
            b'\x08\x04' +
            b'\x08\x05' +
            b'\x08\x06' +
            b'\x08\x07' +
            b'\x09\x08\x01' +
            b'\x00' +

            # then comes the FDE
            b'\x28\x00\x00\x00' +        # length
            b'\x00\x00\x00\x00' +        # CIE_pointer (to CIE at 0)
            b'\x44\x33\x22\x11' +        # initial_location
            b'\x54\x00\x00\x00' +        # address range
            b'\x41' +
            b'\x0e\x0c' + b'\x41' +
            b'\x88\x01' + b'\x41' +
            b'\x86\x02' + b'\x41' +
            b'\x0d\x06' + b'\x41' +
            b'\x84\x03' + b'\x4b' +
            b'\xc4' + b'\x41' +
            b'\xc6' +
            b'\x0d\x07' + b'\x41' +
            b'\xc8' + b'\x41' +
            b'\x0e\x00' +
            b'\x00\x00'
            )
        s.write(data)

        structs = DWARFStructs(little_endian=True, dwarf_format=32, address_size=4)
        cfi = CallFrameInfo(s, len(data), 0, structs)
        entries = cfi.get_entries()

        self.assertEqual(len(entries), 2)
        self.assertIsInstance(entries[0], CIE)
        self.assertEqual(entries[0]['length'], 32)
        self.assertEqual(entries[0]['data_alignment_factor'], -4)
        self.assertEqual(entries[0]['return_address_register'], 8)
        self.assertEqual(len(entries[0].instructions), 11)
        self.assertInstruction(entries[0].instructions[0],
            'DW_CFA_def_cfa', [7, 0])
        self.assertInstruction(entries[0].instructions[8],
            'DW_CFA_same_value', [7])
        self.assertInstruction(entries[0].instructions[9],
            'DW_CFA_register', [8, 1])

        self.assertTrue(isinstance(entries[1], FDE))
        self.assertEqual(entries[1]['length'], 40)
        self.assertEqual(entries[1]['CIE_pointer'], 0)
        self.assertEqual(entries[1]['address_range'], 84)
        self.assertIsNone(entries[1].lsda_pointer)
        self.assertIs(entries[1].cie, entries[0])
        self.assertEqual(len(entries[1].instructions), 21)
        self.assertInstruction(entries[1].instructions[0],
            'DW_CFA_advance_loc', [1])
        self.assertInstruction(entries[1].instructions[1],
            'DW_CFA_def_cfa_offset', [12])
        self.assertInstruction(entries[1].instructions[9],
            'DW_CFA_offset', [4, 3])
        self.assertInstruction(entries[1].instructions[18],
            'DW_CFA_def_cfa_offset', [0])
        self.assertInstruction(entries[1].instructions[20],
            'DW_CFA_nop', [])

        # Now let's decode it...
        decoded_CIE = entries[0].get_decoded()
        self.assertEqual(decoded_CIE.reg_order, list(range(9)))
        self.assertEqual(len(decoded_CIE.table), 1)
        self.assertEqual(decoded_CIE.table[0]['cfa'].reg, 7)
        self.assertEqual(decoded_CIE.table[0]['pc'], 0)
        self.assertEqual(decoded_CIE.table[0]['cfa'].offset, 0)
        self.assertEqual(decoded_CIE.table[0][4].type, RegisterRule.SAME_VALUE)
        self.assertEqual(decoded_CIE.table[0][8].type, RegisterRule.REGISTER)
        self.assertEqual(decoded_CIE.table[0][8].arg, 1)

        decoded_FDE = entries[1].get_decoded()
        self.assertEqual(decoded_FDE.reg_order, list(range(9)))
        self.assertEqual(decoded_FDE.table[0]['cfa'].reg, 7)
        self.assertEqual(decoded_FDE.table[0]['cfa'].offset, 0)
        self.assertEqual(decoded_FDE.table[0]['pc'], 0x11223344)
        self.assertEqual(decoded_FDE.table[0][8].type, RegisterRule.REGISTER)
        self.assertEqual(decoded_FDE.table[0][8].arg, 1)
        self.assertEqual(decoded_FDE.table[1]['cfa'].reg, 7)
        self.assertEqual(decoded_FDE.table[1]['cfa'].offset, 12)
        self.assertEqual(decoded_FDE.table[2][8].type, RegisterRule.OFFSET)
        self.assertEqual(decoded_FDE.table[2][8].arg, -4)
        self.assertEqual(decoded_FDE.table[2][4].type, RegisterRule.SAME_VALUE)
        self.assertEqual(decoded_FDE.table[5]['pc'], 0x11223344 + 20)
        self.assertEqual(decoded_FDE.table[5][4].type, RegisterRule.OFFSET)
        self.assertEqual(decoded_FDE.table[5][4].arg, -12)
        self.assertEqual(decoded_FDE.table[6]['pc'], 0x11223344 + 64)
        self.assertEqual(decoded_FDE.table[9]['pc'], 0x11223344 + 76)

    def test_describe_CFI_instructions(self):
        # The data here represents a single CIE
        data = (b'' +
            b'\x16\x00\x00\x00' +        # length
            b'\xff\xff\xff\xff' +        # CIE_id
            b'\x03\x00\x04\x7c' +        # version, augmentation, caf, daf
            b'\x08' +                    # return address
            b'\x0c\x07\x02' +
            b'\x10\x02\x07\x03\x01\x02\x00\x00\x06\x06')
        s = BytesIO(data)

        structs = DWARFStructs(little_endian=True, dwarf_format=32, address_size=4)
        cfi = CallFrameInfo(s, len(data), 0, structs)
        entries = cfi.get_entries()

        set_global_machine_arch('x86')
        self.assertEqual(describe_CFI_instructions(entries[0]),
            (   '  DW_CFA_def_cfa: r7 (edi) ofs 2\n' +
                '  DW_CFA_expression: r2 (edx) (DW_OP_addr: 201; DW_OP_deref; DW_OP_deref)\n'))

    def test_CFIEntry_get_decoded(self):
        oracle_decoded = DecodedCallFrameTable(
            table = [
                {'pc': 0, 'cfa': CFARule(reg = 29, offset = 0, expr = None)}
            ],
            reg_order = []
        )

        test_dir = join('test', 'testfiles_for_unittests')
        with open(join(test_dir, 'simple_mipsel.elf'), 'rb') as f:
            elf = ELFFile(f)
            di = elf.get_dwarf_info()
            entries = di.CFI_entries()
            decoded = entries[0].get_decoded()
            self.assertEqual(oracle_decoded.table[0]['cfa'].reg,
                decoded.table[0]['cfa'].reg
            )
            self.assertEqual(oracle_decoded.table[0]['cfa'].offset,
                decoded.table[0]['cfa'].offset)

    def test_ehframe_fde_with_lsda_pointer(self):
        # CIE and FDE dumped from exceptions_0, offset 0xcc0
        # binary is at https://github.com/angr/binaries/blob/master/tests/x86_64/exceptions_0
        data = (b'' +
            # CIE
            b'\x1c\x00\x00\x00' +       # length
            b'\x00\x00\x00\x00' +       # ID
            b'\x01' +                   # version
            b'\x7a\x50\x4c\x52\x00' +   # augmentation string
            b'\x01' +                   # code alignment
            b'\x78' +                   # data alignment
            b'\x10' +                   # return address register
            b'\x07' +                   # augmentation data length
            b'\x9b' +                   # personality function pointer encoding
            b'\x3d\x13\x20\x00' +       # personality function pointer
            b'\x1b' +                   # LSDA pointer encoding
            b'\x1b' +                   # FDE encoding
            b'\x0c\x07\x08\x90' +       # initial instructions
            b'\x01\x00\x00' +
            # FDE
            b'\x24\x00\x00\x00' +       # length
            b'\x24\x00\x00\x00' +       # CIE reference pointer
            b'\x62\xfd\xff\xff' +       # pc begin
            b'\x89\x00\x00\x00' +       # pc range
            b'\x04' +                   # augmentation data length
            b'\xb7\x00\x00\x00' +       # LSDA pointer
            b'\x41\x0e\x10\x86' +       # initial instructions
            b'\x02\x43\x0d\x06' +
            b'\x45\x83\x03\x02' +
            b'\x7f\x0c\x07\x08' +
            b'\x00\x00\x00'
            )
        s = BytesIO(data)

        structs = DWARFStructs(little_endian=True, dwarf_format=32, address_size=8)
        cfi = CallFrameInfo(s, len(data), 0, structs, for_eh_frame=True)
        entries = cfi.get_entries()

        self.assertEqual(len(entries), 2)
        self.assertIsInstance(entries[0], CIE)
        self.assertIn('LSDA_encoding', entries[0].augmentation_dict)
        # check LSDA encoding
        lsda_encoding = entries[0].augmentation_dict['LSDA_encoding']
        basic_encoding = lsda_encoding & 0x0f
        modifier = lsda_encoding & 0xf0
        self.assertEqual(basic_encoding, DW_EH_encoding_flags['DW_EH_PE_sdata4'])
        self.assertEqual(modifier, DW_EH_encoding_flags['DW_EH_PE_pcrel'])
        self.assertIsInstance(entries[1], FDE)
        self.assertEqual(entries[1].lsda_pointer, 232)

if __name__ == '__main__':
    unittest.main()



#-------------------------------------------------------------------------------
# elftools tests
#
# Ricardo Barbedo (ricardo@barbedo.me)
# This code is in the public domain
#-------------------------------------------------------------------------------
import unittest
import os

from elftools.elf.elffile import ELFFile

class TestRISCVSupport(unittest.TestCase):
    def test_hello(self):
        with open(os.path.join('test', 'testfiles_for_unittests',
                               'simple_gcc.elf.riscv'), 'rb') as f:
            elf = ELFFile(f)
            self.assertEqual(elf.get_machine_arch(), 'RISC-V')

            # Check some other properties of this ELF file derived from readelf
            self.assertEqual(elf['e_entry'], 0x10116)
            self.assertEqual(elf.num_sections(), 13)
            self.assertEqual(elf.num_segments(), 3)

    def test_build_attributes(self):
        with open(os.path.join('test', 'testfiles_for_unittests',
                               'simple_gcc.elf.riscv'), 'rb') as f:
            elf = ELFFile(f)

            sec = elf.get_section_by_name('.riscv.attributes')
            self.assertEqual(sec['sh_type'], 'SHT_RISCV_ATTRIBUTES')
            self.assertEqual(sec.num_subsections, 1)

            subsec = sec.subsections[0]
            self.assertEqual(subsec.header['vendor_name'], 'riscv')
            self.assertEqual(subsec.num_subsubsections, 1)

            subsubsec = subsec.subsubsections[0]
            self.assertEqual(subsubsec.header.tag, 'TAG_FILE')

            for i in subsubsec.iter_attributes('TAG_STACK_ALIGN'):
                self.assertEqual(i.value, 16)

            for i in subsubsec.iter_attributes('TAG_ARCH'):
                self.assertEqual(i.value, 'rv64i2p0_m2p0_a2p0_f2p0_d2p0_c2p0_v1p0_zfh1p0_zfhmin1p0_zba1p0_zbb1p0_zbc1p0_zbs1p0_zve32f1p0_zve32x1p0_zve64d1p0_zve64f1p0_zve64x1p0_zvl128b1p0_zvl32b1p0_zvl64b1p0')

        with open(os.path.join('test', 'testfiles_for_unittests',
                               'simple_clang.elf.riscv'), 'rb') as f:
            elf = ELFFile(f)

            sec = elf.get_section_by_name('.riscv.attributes')
            self.assertEqual(sec['sh_type'], 'SHT_RISCV_ATTRIBUTES')
            self.assertEqual(sec.num_subsections, 1)

            subsec = sec.subsections[0]
            self.assertEqual(subsec.header['vendor_name'], 'riscv')
            self.assertEqual(subsec.num_subsubsections, 1)

            subsubsec = subsec.subsubsections[0]
            self.assertEqual(subsubsec.header.tag, 'TAG_FILE')

            for i in subsubsec.iter_attributes('TAG_STACK_ALIGN'):
                self.assertEqual(i.value, 16)

            for i in subsubsec.iter_attributes('TAG_ARCH'):
                self.assertEqual(i.value, 'rv64i2p1_m2p0_a2p1_f2p2_d2p2_c2p0_v1p0_zicsr2p0_zifencei2p0_zmmul1p0_zfh1p0_zba1p0_zbb1p0_zbc1p0_zbs1p0_zve32f1p0_zve32x1p0_zve64d1p0_zve64f1p0_zve64x1p0_zvl128b1p0_zvl32b1p0_zvl64b1p0')

            for i in subsubsec.iter_attributes('TAG_PRIV_SPEC'):
                self.assertEqual(i.value, 1)

            for i in subsubsec.iter_attributes('TAG_PRIV_SPEC_MINOR'):
                self.assertEqual(i.value, 11)

if __name__ == '__main__':
    unittest.main()



#------------------------------------------------------------------------------
# elftools tests
#
# Maxim Akhmedov (max42@yandex-team.ru)
# This code is in the public domain
#------------------------------------------------------------------------------
import unittest
import os

from elftools.elf.elffile import ELFFile
from elftools.elf.segments import NoteSegment


class TestCoreNotes(unittest.TestCase):
    """ This test makes sure than core dump specific
        sections are properly analyzed.
    """
    @classmethod
    def setUpClass(cls):
        cls._core_file = open(os.path.join('test',
                              'testfiles_for_unittests', 'core_linux64.elf'),
                              'rb')

    def test_core_prpsinfo(self):
        elf = ELFFile(self._core_file)
        for segment in elf.iter_segments():
            if not isinstance(segment, NoteSegment):
                continue
            notes = list(segment.iter_notes())
            for note in segment.iter_notes():
                if note['n_type'] != 'NT_PRPSINFO':
                    continue
                desc = note['n_desc']
                self.assertEqual(desc['pr_state'], 0)
                self.assertEqual(desc['pr_sname'], b'R')
                self.assertEqual(desc['pr_zomb'], 0)
                self.assertEqual(desc['pr_nice'], 0)
                self.assertEqual(desc['pr_flag'], 0x400600)
                self.assertEqual(desc['pr_uid'], 1000)
                self.assertEqual(desc['pr_gid'], 1000)
                self.assertEqual(desc['pr_pid'], 23395)
                self.assertEqual(desc['pr_ppid'], 23187)
                self.assertEqual(desc['pr_pgrp'], 23395)
                self.assertEqual(desc['pr_sid'], 23187)
                self.assertEqual(
                    desc['pr_fname'],
                    b'coredump_self\x00\x00\x00')
                self.assertEqual(
                    desc['pr_psargs'],
                    b'./coredump_self foo bar 42 ' + b'\x00' * (80 - 27))

    def test_core_nt_file(self):
        """
        Test that the parsing of the NT_FILE note within a core file is
        correct.
        The assertions are made against the output of eu-readelf.

        eu-readelf -n core_linux64.elf
        ...
        CORE                 621  FILE
        10 files:
        00400000-00401000 00000000 4096
          /home/max42/pyelftools/test/coredump_self
        00600000-00601000 00000000 4096
          /home/max42/pyelftools/test/coredump_self
        00601000-00602000 00001000 4096
          /home/max42/pyelftools/test/coredump_self
        7fa4593ae000-7fa45956d000 00000000 1830912
          /lib/x86_64-linux-gnu/libc-2.23.so
        7fa45956d000-7fa45976d000 001bf000 2097152
          /lib/x86_64-linux-gnu/libc-2.23.so
        7fa45976d000-7fa459771000 001bf000 16384
          /lib/x86_64-linux-gnu/libc-2.23.so
        7fa459771000-7fa459773000 001c3000 8192
          /lib/x86_64-linux-gnu/libc-2.23.so
        7fa459777000-7fa45979d000 00000000 155648
          /lib/x86_64-linux-gnu/ld-2.23.so
        7fa45999c000-7fa45999d000 00025000 4096
          /lib/x86_64-linux-gnu/ld-2.23.so
        7fa45999d000-7fa45999e000 00026000 4096
          /lib/x86_64-linux-gnu/ld-2.23.so
        ...
        """
        elf = ELFFile(self._core_file)
        nt_file_found = False
        for segment in elf.iter_segments():
            if not isinstance(segment, NoteSegment):
                continue
            for note in segment.iter_notes():
                if note['n_type'] != 'NT_FILE':
                    continue
                nt_file_found = True
                desc = note['n_desc']
                self.assertEqual(desc['num_map_entries'], 10)
                self.assertEqual(desc['page_size'], 4096)
                self.assertEqual(len(desc['Elf_Nt_File_Entry']), 10)
                self.assertEqual(len(desc['filename']), 10)

                self.validate_nt_file_entry(desc['Elf_Nt_File_Entry'][0],
                                            desc['page_size'],
                                            0x00400000,
                                            0x00401000,
                                            0x00000000)
                self.assertEqual(desc['filename'][0],
                                 b"/home/max42/pyelftools/test/coredump_self")

                self.validate_nt_file_entry(desc['Elf_Nt_File_Entry'][1],
                                            desc['page_size'],
                                            0x00600000,
                                            0x00601000,
                                            0x00000000)
                self.assertEqual(desc['filename'][1],
                                 b"/home/max42/pyelftools/test/coredump_self")

                self.validate_nt_file_entry(desc['Elf_Nt_File_Entry'][2],
                                            desc['page_size'],
                                            0x00601000,
                                            0x00602000,
                                            0x00001000)
                self.assertEqual(desc['filename'][2],
                                 b"/home/max42/pyelftools/test/coredump_self")

                self.validate_nt_file_entry(desc['Elf_Nt_File_Entry'][3],
                                            desc['page_size'],
                                            0x7fa4593ae000,
                                            0x7fa45956d000,
                                            0x00000000)
                self.assertEqual(desc['filename'][3],
                                 b"/lib/x86_64-linux-gnu/libc-2.23.so")

                self.validate_nt_file_entry(desc['Elf_Nt_File_Entry'][4],
                                            desc['page_size'],
                                            0x7fa45956d000,
                                            0x7fa45976d000,
                                            0x001bf000)
                self.assertEqual(desc['filename'][4],
                                 b"/lib/x86_64-linux-gnu/libc-2.23.so")

                self.validate_nt_file_entry(desc['Elf_Nt_File_Entry'][5],
                                            desc['page_size'],
                                            0x7fa45976d000,
                                            0x7fa459771000,
                                            0x001bf000)
                self.assertEqual(desc['filename'][5],
                                 b"/lib/x86_64-linux-gnu/libc-2.23.so")

                self.validate_nt_file_entry(desc['Elf_Nt_File_Entry'][6],
                                            desc['page_size'],
                                            0x7fa459771000,
                                            0x7fa459773000,
                                            0x001c3000)
                self.assertEqual(desc['filename'][6],
                                 b"/lib/x86_64-linux-gnu/libc-2.23.so")

                self.validate_nt_file_entry(desc['Elf_Nt_File_Entry'][7],
                                            desc['page_size'],
                                            0x7fa459777000,
                                            0x7fa45979d000,
                                            0x00000000)
                self.assertEqual(desc['filename'][7],
                                 b"/lib/x86_64-linux-gnu/ld-2.23.so")

                self.validate_nt_file_entry(desc['Elf_Nt_File_Entry'][8],
                                            desc['page_size'],
                                            0x7fa45999c000,
                                            0x7fa45999d000,
                                            0x00025000)
                self.assertEqual(desc['filename'][8],
                                 b"/lib/x86_64-linux-gnu/ld-2.23.so")

                self.validate_nt_file_entry(desc['Elf_Nt_File_Entry'][9],
                                            desc['page_size'],
                                            0x7fa45999d000,
                                            0x7fa45999e000,
                                            0x00026000)
                self.assertEqual(desc['filename'][9],
                                 b"/lib/x86_64-linux-gnu/ld-2.23.so")
        self.assertTrue(nt_file_found)

    def validate_nt_file_entry(self,
                               entry,
                               page_size,
                               expected_vm_start,
                               expected_vm_end,
                               expected_page_offset):
        self.assertEqual(entry.vm_start, expected_vm_start)
        self.assertEqual(entry.vm_end, expected_vm_end)
        self.assertEqual(entry.page_offset * page_size, expected_page_offset)

    @classmethod
    def tearDownClass(cls):
        cls._core_file.close()



import os
from io import BytesIO
import sys
import unittest

from elftools.elf.elffile import ELFFile
from elftools.elf.dynamic import DynamicSegment, DynamicSection


class TestRelocation(unittest.TestCase):
    def test_dynamic_segment(self):
        """Verify that we can process relocations on the PT_DYNAMIC segment without section headers"""

        test_dir = os.path.join('test', 'testfiles_for_unittests')
        with open(os.path.join(test_dir, 'x64_bad_sections.elf'), 'rb') as f:
            elff = ELFFile(f)

            for seg in elff.iter_segments():
                if isinstance(seg, DynamicSegment):
                    relos = seg.get_relocation_tables()
                    self.assertEqual(set(relos), {'JMPREL', 'RELA'})

    def test_dynamic_section(self):
        """Verify that we can parse relocations from the .dynamic section"""

        test_dir = os.path.join('test', 'testfiles_for_unittests')
        with open(os.path.join(test_dir, 'sample_exe64.elf'), 'rb') as f:
            elff = ELFFile(f)

            for sect in elff.iter_sections():
                if isinstance(sect, DynamicSection):
                    relos = sect.get_relocation_tables()
                    self.assertEqual(set(relos), {'JMPREL', 'RELA'})

    def test_dynamic_section_solaris(self):
        """Verify that we can parse relocations from the .dynamic section"""

        test_dir = os.path.join('test', 'testfiles_for_unittests')
        with open(os.path.join(test_dir, 'exe_solaris32_cc.elf'), 'rb') as f:
            elff = ELFFile(f)

            for sect in elff.iter_sections():
                if isinstance(sect, DynamicSection):
                    relos = sect.get_relocation_tables()
                    self.assertEqual(set(relos), {'JMPREL', 'REL'})

if __name__ == '__main__':
    unittest.main()



#-------------------------------------------------------------------------------
# elftools example: elf_notes.py
#
# An example of obtaining note sections from an ELF file and examining
# the notes it contains.
#
# Eli Bendersky (eliben@gmail.com)
# This code is in the public domain
#-------------------------------------------------------------------------------
from __future__ import print_function
import sys

# If pyelftools is not installed, the example can also run from the root or
# examples/ dir of the source distribution.
sys.path[0:0] = ['.', '..']

from elftools.elf.elffile import ELFFile
from elftools.elf.sections import NoteSection
from elftools.common.utils import bytes2hex


def process_file(filename):
    print('Processing file:', filename)
    with open(filename, 'rb') as f:
        for sect in ELFFile(f).iter_sections():
            if not isinstance(sect, NoteSection):
                continue
            print('  Note section "%s" at offset 0x%.8x with size %d' % (
                sect.name, sect.header['sh_offset'], sect.header['sh_size']))
            for note in sect.iter_notes():
                print('    Name:', note['n_name'])
                print('    Type:', note['n_type'])
                desc = note['n_desc']
                if note['n_type'] == 'NT_GNU_ABI_TAG':
                    print('    Desc: %s, ABI: %d.%d.%d' % (
                        desc['abi_os'],
                        desc['abi_major'],
                        desc['abi_minor'],
                        desc['abi_tiny']))
                elif note['n_type'] in {'NT_GNU_BUILD_ID', 'NT_GNU_GOLD_VERSION'}:
                    print('    Desc:', desc)
                else:
                    print('    Desc:', bytes2hex(desc))


if __name__ == '__main__':
    if sys.argv[1] == '--test':
        for filename in sys.argv[2:]:
            process_file(filename)



#-------------------------------------------------------------------------------
# elftools tests
#
# Eli Bendersky (eliben@gmail.com)
# This code is in the public domain
#-------------------------------------------------------------------------------
import unittest
import os

from elftools.elf.elffile import ELFFile
from elftools.elf.sections import StabSection


class TestStab(unittest.TestCase):
    def test_stab(self):
        expected = [
            ("obj_stabs.S", 0, 0, 0x2, 33), # generated by compiler
            ("label", 0x95, 0xc8, 0x4072, 0xdeadbeef),
            ("another label", 0x41, 0x66, 0xf9b1, 0xcafebabe)]
        with open(os.path.join('test', 'testfiles_for_unittests',
                               'obj_stabs.elf'), 'rb') as f:
            elf = ELFFile(f)

            # using correct type?
            for s in elf.iter_sections():
                if s.name == '.stab':
                    self.assertIsInstance(s, StabSection)

            # check section contents
            stab = elf.get_section_by_name('.stab')
            stabstr = elf.get_section_by_name('.stabstr')
            for entry, golden in zip(stab.iter_stabs(), expected):
                self.assertEqual(stabstr.get_string(entry.n_strx), golden[0])
                self.assertEqual(entry.n_type, golden[1])
                self.assertEqual(entry.n_other, golden[2])
                self.assertEqual(entry.n_desc, golden[3])
                self.assertEqual(entry.n_value, golden[4])


if __name__ == '__main__':
    unittest.main()



"""
Test that elftools does not fail to load debug symbol ELF files
"""
import unittest
import os

from elftools.elf.elffile import ELFFile, DynamicSection
from elftools.dwarf.callframe import ZERO

class TestDBGFile(unittest.TestCase):
    def test_dynamic_segment(self):
        """ Test that the degenerate case for the dynamic segment does not crash
        """
        with open(os.path.join('test', 'testfiles_for_unittests',
                               'debug_info.elf'), 'rb') as f:
            elf = ELFFile(f)

            seen_dynamic_segment = False
            for segment in elf.iter_segments():
                if segment.header.p_type == 'PT_DYNAMIC':
                    self.assertEqual(segment.num_tags(), 0, "The dynamic segment in this file should be empty")
                    seen_dynamic_segment = True
                    break

            self.assertTrue(seen_dynamic_segment, "There should be a dynamic segment in this file")

    def test_dynamic_section(self):
        """ Test that the degenerate case for the dynamic section does not crash
        """
        with open(os.path.join('test', 'testfiles_for_unittests',
                               'debug_info.elf'), 'rb') as f:
            elf = ELFFile(f)
            section = DynamicSection(elf.get_section_by_name('.dynamic').header, '.dynamic', elf)

            self.assertEqual(section.num_tags(), 0, "The dynamic section in this file should be empty")

    def test_eh_frame(self):
        """ Test that parsing .eh_frame with SHT_NOBITS does not crash
        """
        with open(os.path.join('test', 'testfiles_for_unittests',
                               'debug_info.elf'), 'rb') as f:
            elf = ELFFile(f)
            dwarf = elf.get_dwarf_info()
            eh_frame = list(dwarf.EH_CFI_entries())
            self.assertEqual(len(eh_frame), 1, "There should only be the ZERO entry in eh_frame")
            self.assertIs(type(eh_frame[0]), ZERO, "The only eh_frame entry should be the terminator")

if __name__ == '__main__':
    unittest.main()



#------------------------------------------------------------------------------
# elftools tests
#
# Gabriele Digregorio - Io_no
# This code is in the public domain
#------------------------------------------------------------------------------

from elftools.elf.elffile import ELFFile
import unittest


class TestDebuglink(unittest.TestCase):
    """ This test verifies that the .gnu_debuglink section is followed and parsed correctly.
        The test file contains a .gnu_debuglink section that points to a debug file
        containing DWARF info.
        We verify that the subprograms are correctly retrieved from the debug file.
    """

    def stream_loader(self, external_filename: str) -> 'IO[bytes]':
        """
        This function takes an external filename to load a supplementary object file,
        and returns a stream suitable for creating a new ELFFile.
        Args:
            external_filename (str): The relative file path to load.

        Returns:
            stream (IO[bytes]): A stream suitable for creating a new ELFFile.
        """
        self.assertEqual(external_filename, b'debuglink.debug')
        stream = open(b'test/testfiles_for_unittests/' + external_filename, 'rb')
        return stream

    def subprograms_from_debuglink(self, elf: ELFFile) -> dict[str, (int, int)]:
        """Returns a dictionary containing the subprograms of the specified ELF file from the linked
        debug file.
        Args:
            elf (ELFFile): The ELF file.

        Returns:
            dict: A dictionary containing the subprograms of the specified ELF file.
        """
        subprograms = {}

        # Retrieve the subprograms from the DWARF info
        dwarf_info = elf.get_dwarf_info(follow_links=True, relocate_dwarf_sections=True)

        if dwarf_info:
            for CU in dwarf_info.iter_CUs():
                for DIE in CU.iter_DIEs():
                    if DIE.tag == 'DW_TAG_subprogram':
                        attributes = DIE.attributes
                        lowpc_attr = attributes.get('DW_AT_low_pc')
                        highpc_attr = attributes.get('DW_AT_high_pc')
                        name_attr = attributes.get('DW_AT_name')
                        if not lowpc_attr or not highpc_attr or not name_attr:
                            continue
                        lowpc = lowpc_attr.value
                        if highpc_attr.form == 'DW_FORM_addr':
                            # highpc is an absolute address
                            size = highpc_attr.value - lowpc
                        elif highpc_attr.form in {'DW_FORM_data2','DW_FORM_data4',
                                                    'DW_FORM_data8', 'DW_FORM_data1',
                                                    'DW_FORM_udata'}:
                            # highpc is an offset from lowpc
                            size = highpc_attr.value
                        name = name_attr.value
                        subprograms[name] = (lowpc, size)
        return subprograms

    def test_debuglink(self):
        with open('test/testfiles_for_unittests/debuglink', "rb") as elf_file:
            elf = ELFFile(elf_file, stream_loader=self.stream_loader)
            # Contains eh_frame and gnu_debuglink, but no DWARF
            self.assertTrue(elf.has_dwarf_info(False))
            self.assertFalse(elf.has_dwarf_info(True))
            self.assertTrue(elf.has_dwarf_link())

            link = elf.get_dwarf_link()
            self.assertIsNotNone(link)
            self.assertEqual(link.filename, b'debuglink.debug')
            self.assertEqual(link.checksum, 0x29b7c5f1)

            subprograms = self.subprograms_from_debuglink(elf)
            self.assertEqual(subprograms, {b'main': (0x1161, 0x52), b'addNumbers': (0x1149, 0x18)})

        # Test the filesystem aware ELFFile loading
        elf = ELFFile.load_from_path('test/testfiles_for_unittests/debuglink')
        subprograms = self.subprograms_from_debuglink(elf)
        self.assertEqual(subprograms, {b'main': (0x1161, 0x52), b'addNumbers': (0x1149, 0x18)})

if __name__ == '__main__':
    unittest.main()




import os
import unittest

from elftools.elf.elffile import ELFFile
from elftools.elf.sections import NoteSection


class TestNotes(unittest.TestCase):
    def test_note_after_gnu_property_type_note(self):
        with ELFFile.load_from_path(os.path.join('test', 'testfiles_for_unittests', 'note_after_gnu_property', 'main.elf')) as elf:
            note_sections = [section for section in elf.iter_sections() if isinstance(section, NoteSection)]
            # There's only one note section in this file:
            self.assertEqual(len(note_sections), 1)
            notes = list(note_sections[0].iter_notes())
            # There are 2 notes in this section:
            self.assertEqual(len(notes), 2)
            # The first note is the GNU_PROPERTY_TYPE_0 note:
            self.assertEqual(notes[0].n_type, 'NT_GNU_PROPERTY_TYPE_0')
            # It should only have two Elf_Props (and not attempt to parse the note after it as Elf_Props):
            self.assertEqual(len(notes[0].n_desc), 2)

    def test_note_segment_with_8_byte_alignment(self):
        with ELFFile.load_from_path(os.path.join('test', 'testfiles_for_unittests', 'note_with_segment_padding', 'main.elf')) as elf:
            note_sections = [section for section in elf.iter_sections() if isinstance(section, NoteSection)]
            # There's only one note section in this file:
            self.assertEqual(len(note_sections), 1)
            notes = list(note_sections[0].iter_notes())
            # There's one note in this section:
            self.assertEqual(len(notes), 1)

    def test_note_tc3xx_blinky(self):
        with ELFFile.load_from_path(os.path.join('test', 'testfiles_for_unittests', 'note_tc3xxx_blinky.elf')) as elf:
            note_sections = [section for section in elf.iter_sections() if isinstance(section, NoteSection)]
            # There's only one note section in this file:
            self.assertEqual(len(note_sections), 1)
            notes = list(note_sections[0].iter_notes())
            # There's one note in this section:
            self.assertEqual(len(notes), 522)



#-------------------------------------------------------------------------------
# elftools example: dwarf_pubnames_types.py
#
# Dump the contents of .debug_pubnames and .debug_pubtypes sections from the
# ELF file.
#
# Note: sample_exe64.elf doesn't have a .debug_pubtypes section.
#
# Vijay Ramasami (rvijayc@gmail.com)
# This code is in the public domain
#-------------------------------------------------------------------------------
from __future__ import print_function
import sys

# If pyelftools is not installed, the example can also run from the root or
# examples/ dir of the source distribution.
sys.path[0:0] = ['.', '..']

from elftools.elf.elffile import ELFFile
from elftools.common.utils import bytes2str

def process_file(filename):
    print('Processing file:', filename)
    with open(filename, 'rb') as f:
        elffile = ELFFile(f)

        if not elffile.has_dwarf_info():
            print('  file has no DWARF info')
            return

        # get_dwarf_info returns a DWARFInfo context object, which is the
        # starting point for all DWARF-based processing in pyelftools.
        dwarfinfo = elffile.get_dwarf_info()

        # get .debug_pubtypes section.
        pubnames = dwarfinfo.get_pubnames()
        if pubnames is None:
            print('ERROR: No .debug_pubnames section found in ELF.')
        else:
            print('%d entries found in .debug_pubnames' % len(pubnames))

            print('Trying pubnames example ...')
            for name, entry in pubnames.items():
                print('%s: cu_ofs = %d, die_ofs = %d' %
                        (name, entry.cu_ofs, entry.die_ofs))

                # get the actual CU/DIE that has this information.
                print('Fetching the actual die for %s ...' % name)
                for cu in dwarfinfo.iter_CUs():
                    if cu.cu_offset == entry.cu_ofs:
                        for die in cu.iter_DIEs():
                            if die.offset == entry.die_ofs:
                                print('Die Name: %s' %
                                        bytes2str(die.attributes['DW_AT_name'].value))

            # dump all entries in .debug_pubnames section.
            print('Dumping .debug_pubnames table ...')
            print('-' * 66)
            print('%50s%8s%8s' % ('Symbol', 'CU_OFS', 'DIE_OFS'))
            print('-' * 66)
            for (name, entry) in pubnames.items():
                print('%50s%8d%8d' % (name, entry.cu_ofs, entry.die_ofs))
            print('-' * 66)

        # get .debug_pubtypes section.
        pubtypes = dwarfinfo.get_pubtypes()
        if pubtypes is None:
            print('ERROR: No .debug_pubtypes section found in ELF')
        else:
            print('%d entries found in .debug_pubtypes' % len(pubtypes))

            for name, entry in pubtypes.items():
                print('%s: cu_ofs = %d, die_ofs = %d' %
                        (name, entry.cu_ofs, entry.die_ofs))

                # get the actual CU/DIE that has this information.
                print('Fetching the actual die for %s ...' % name)
                for cu in dwarfinfo.iter_CUs():
                    if cu.cu_offset == entry.cu_ofs:
                        for die in cu.iter_DIEs():
                            if die.offset == entry.die_ofs:
                                print('Die Name: %s' %
                                        bytes2str(die.attributes['DW_AT_name'].value))
                                die_info_rec(die)

            # dump all entries in .debug_pubtypes section.
            print('Dumping .debug_pubtypes table ...')
            print('-' * 66)
            print('%50s%8s%8s' % ('Symbol', 'CU_OFS', 'DIE_OFS'))
            print('-' * 66)
            for (name, entry) in pubtypes.items():
                print('%50s%8d%8d' % (name, entry.cu_ofs, entry.die_ofs))
            print('-' * 66)


def die_info_rec(die, indent_level='    '):
    """ A recursive function for showing information about a DIE and its
        children.
    """
    print(indent_level + 'DIE tag=%s, attrs=' % die.tag)
    for name, val in die.attributes.items():
        print(indent_level + '  %s = %s' % (name, val))
    child_indent = indent_level + '  '
    for child in die.iter_children():
        die_info_rec(child, child_indent)


if __name__ == '__main__':
    if sys.argv[1] == '--test':
        process_file(sys.argv[2])
        sys.exit(0)

    if len(sys.argv) < 2:
        print('Expected usage: {0} <executable>'.format(sys.argv[0]))
        sys.exit(1)
    process_file(sys.argv[1])



#-------------------------------------------------------------------------------
# elftools tests
#
# Test 'R_ARM_CALL' relocation type support.
# Compare the '.text' section data of ELF file that was relocated by elftools
# with an ELF file that was relocated by linker.
#
# Dmitry Koltunov (koltunov@ispras.ru)
# This code is in the public domain
#-------------------------------------------------------------------------------
import os
from io import BytesIO
import sys
import unittest

from elftools.elf.elffile import ELFFile
from elftools.elf.relocation import RelocationHandler


def do_relocation(rel_elf):
    data = rel_elf.get_section_by_name('.text').data()
    rh = RelocationHandler(rel_elf)

    stream = BytesIO()
    stream.write(data)

    rel = rel_elf.get_section_by_name('.rel.text')
    rh.apply_section_relocations(stream, rel)
    return stream.getvalue()


class TestARMRElocation(unittest.TestCase):
    def test_reloc(self):
        test_dir = os.path.join('test', 'testfiles_for_unittests')
        with open(os.path.join(test_dir, 'arm_reloc_unrelocated.o'), 'rb') as rel_f, \
             open(os.path.join(test_dir, 'arm_reloc_relocated.elf'), 'rb') as f:
            rel_elf = ELFFile(rel_f)
            elf = ELFFile(f)

            # Comparison of '.text' section data
            self.assertEqual(do_relocation(rel_elf),
                              elf.get_section_by_name('.text').data())

if __name__ == '__main__':
    unittest.main()



#------------------------------------------------------------------------------
# elftools tests
#
# Yann Rouillard (yann@pleiades.fr.eu.org)
# This code is in the public domain
#------------------------------------------------------------------------------
import unittest
import os

from elftools.elf.elffile import ELFFile
from elftools.elf.dynamic import DynamicSection, DynamicTag


class TestDoubleDynstrSections(unittest.TestCase):
    """ This test make sure than dynamic tags
        are properly analyzed when two .dynstr
        sections are present in an elf file
    """

    reference_data = [
        'libz.so.1',
        'libc.so.6',
        'lib_versioned.so.1',
    ]

    def _test_double_dynstr_section_generic(self, testfile):

        with open(os.path.join('test', 'testfiles_for_unittests', testfile),
                  'rb') as f:
            elf = ELFFile(f)
            for section in elf.iter_sections():
                if isinstance(section, DynamicSection):
                    d_tags = [getattr(x, x.entry.d_tag[3:].lower())
                              for x in section.iter_tags()
                              if x.entry.d_tag in DynamicTag._HANDLED_TAGS]
                    self.assertListEqual(
                            TestDoubleDynstrSections.reference_data,
                            d_tags)
                    return
            self.fail('No dynamic section found !!')


    def test_double_dynstr_section(self):
        """ First test with the good dynstr section first
        """
        self._test_double_dynstr_section_generic(
                'lib_with_two_dynstr_sections.so.1.elf')

    def test_double_dynstr_section_reverse(self):
        """ Second test with the good dynstr section last
        """
        self._test_double_dynstr_section_generic(
                'lib_with_two_dynstr_sections_reversed.so.1.elf')


if __name__ == '__main__':
    unittest.main()



#-------------------------------------------------------------------------------
# pyelftools: setup.py
#
# Setup/installation script.
#
# Eli Bendersky (eliben@gmail.com)
# This code is in the public domain
#-------------------------------------------------------------------------------
import os, sys
from setuptools import setup


try:
    with open('README', 'rt') as readme:
        description = '\n' + readme.read()
except IOError:
    # maybe running setup.py from some other dir
    description = ''


setup(
    # metadata
    name='pyelftools',
    description='Library for analyzing ELF files and DWARF debugging information',
    long_description=description,
    license='Public domain',
    version='0.32',
    author='Eli Bendersky',
    maintainer='Eli Bendersky',
    author_email='eliben@gmail.com',
    url='https://github.com/eliben/pyelftools',
    platforms='Cross Platform',
    classifiers = [
        'Programming Language :: Python :: 3',
        ],

    # All packages and sub-packages must be listed here
    packages=[
        'elftools',
        'elftools.elf',
        'elftools.common',
        'elftools.dwarf',
        'elftools.ehabi',
        'elftools.construct', 'elftools.construct.lib',
        ],

    scripts=['scripts/readelf.py'],
    package_data={'elftools': ['py.typed']}
)



import os
import unittest

from elftools.elf.elffile import ELFFile

address_a = 0x112f;
address_b = 0x1154;

class TestRangeLists(unittest.TestCase):
    def test_arange_absent(self):
        with open(os.path.join('test', 'testfiles_for_unittests', 'aranges_absent.elf'), 'rb') as f:
            elffile = ELFFile(f)
            self.assertTrue(elffile.has_dwarf_info())
            aranges = elffile.get_dwarf_info().get_aranges()
            self.assertIsNone(aranges)

    def test_arange_partial(self):
        with open(os.path.join('test', 'testfiles_for_unittests', 'aranges_partial.elf'), 'rb') as f:
            elffile = ELFFile(f)
            self.assertTrue(elffile.has_dwarf_info())
            aranges = elffile.get_dwarf_info().get_aranges()
            self.assertIsNotNone(aranges)
            self.assertIsNone(aranges.cu_offset_at_addr(address_a))
            self.assertIsNotNone(aranges.cu_offset_at_addr(address_b))

    def test_arange_complete(self):
        with open(os.path.join('test', 'testfiles_for_unittests', 'aranges_complete.elf'), 'rb') as f:
            elffile = ELFFile(f)
            self.assertTrue(elffile.has_dwarf_info())
            aranges = elffile.get_dwarf_info().get_aranges()
            self.assertIsNotNone(aranges)
            self.assertIsNotNone(aranges.cu_offset_at_addr(address_a))
            self.assertIsNotNone(aranges.cu_offset_at_addr(address_b))

if __name__ == '__main__':
    unittest.main()



#-------------------------------------------------------------------------------
# Test handling for compressed debug sections
#
# Pierre-Marie de Rodat (pmderodat@kawie.fr)
# This code is in the public domain
#-------------------------------------------------------------------------------

from contextlib import contextmanager
import os
import unittest

from elftools.elf.elffile import ELFFile
from elftools.common.exceptions import ELFCompressionError


class TestCompressedSupport(unittest.TestCase):

    def test_compressed_32(self):
        with self.elffile('32') as elf:
            section = elf.get_section_by_name('.debug_info')
            self.assertTrue(section.compressed)
            self.assertEqual(section.data_size, 0x330)
            self.assertEqual(section.data_alignment, 1)

            self.assertEqual(self.get_cus_info(elf), ['CU 0x0: 0xb-0x322'])

    def test_compressed_64(self):
        with self.elffile('64') as elf:
            section = elf.get_section_by_name('.debug_info')
            self.assertTrue(section.compressed)
            self.assertEqual(section.data_size, 0x327)
            self.assertEqual(section.data_alignment, 1)
            self.assertEqual(self.get_cus_info(elf), ['CU 0x0: 0xb-0x319'])

    def test_compressed_unknown_type(self):
        with self.elffile('unknown_type') as elf:
            section = elf.get_section_by_name('.debug_info')
            try:
                section.data()
            except ELFCompressionError as exc:
                self.assertEqual(
                    str(exc), 'Unknown compression type: 0x7ffffffe'
                )
            else:
                self.fail('An exception was exected')

    def test_compressed_bad_size(self):
        with self.elffile('bad_size') as elf:
            section = elf.get_section_by_name('.debug_info')
            try:
                section.data()
            except ELFCompressionError as exc:
                self.assertEqual(
                    str(exc),
                    'Decompressed data is 807 bytes long, should be 808 bytes'
                    ' long'
                )
            else:
                self.fail('An exception was exected')

    # Test helpers

    @contextmanager
    def elffile(self, name):
        """ Context manager to open and parse an ELF file.
        """
        with open(os.path.join('test', 'testfiles_for_unittests',
                               'compressed_{}.o'.format(name)), 'rb') as f:
            yield ELFFile(f)

    def get_cus_info(self, elffile):
        """ Return basic info about the compile units in `elffile`.

        This is used as a basic sanity check for decompressed DWARF data.
        """
        result = []

        dwarf = elffile.get_dwarf_info()
        for cu in dwarf.iter_CUs():
            dies = []

            def traverse(die):
                dies.append(die.offset)
                for child in die.iter_children():
                    traverse(child)

            traverse(cu.get_top_DIE())
            result.append('CU {:#0x}: {:#0x}-{:#0x}'.format(
                cu.cu_offset,
                dies[0], dies[-1]
            ))

        return result



#-------------------------------------------------------------------------------
# Tests the functionality of get_section_index
#
# Jonathan Bruchim (YonBruchim@gmail.com)
# This code is in the public domain
#-------------------------------------------------------------------------------
import unittest
import os

from elftools.elf.elffile import ELFFile


class TestGetSectionIndex(unittest.TestCase):
    def test_existing_section(self):
        with open(os.path.join('test', 'testfiles_for_unittests',
                               'simple_gcc.elf.arm'), 'rb') as f:
            elf = ELFFile(f)

            # Find the symbol table.
            data_section_index = elf.get_section_index('.data')
            self.assertIsNotNone(data_section_index)

            # Test we can find a symbol by its name.
            data_section = elf.get_section(data_section_index)
            self.assertIsNotNone(data_section)

            # Test it is actually the symbol we expect.
            self.assertEqual(data_section.name, '.data')

    def test_missing_section(self):
        with open(os.path.join('test', 'testfiles_for_unittests',
                               'simple_gcc.elf.arm'), 'rb') as f:
            elf = ELFFile(f)

            # try getting a missing section index
            missing_section_index = elf.get_section_index('non-existent section')
            self.assertIsNone(missing_section_index)

    def test_uninitialized_section_name_map(self):
        with open(os.path.join('test', 'testfiles_for_unittests',
                               'simple_gcc.elf.arm'), 'rb') as f:
            elf = ELFFile(f)

            elf._section_name_map = None

            # Find the symbol table.
            data_section_index = elf.get_section_index('.data')
            self.assertIsNotNone(data_section_index)

            # Test we can find a symbol by its name.
            data_section = elf.get_section(data_section_index)
            self.assertIsNotNone(data_section)

            # Test it is actually the symbol we expect.
            self.assertEqual(data_section.name, '.data')


if __name__ == '__main__':
    unittest.main()




#-------------------------------------------------------------------------------
# elftools tests
#
# Eli Bendersky (eliben@gmail.com)
# This code is in the public domain
#-------------------------------------------------------------------------------
import unittest

from elftools.dwarf.descriptions import ExprDumper, set_global_machine_arch
from elftools.dwarf.dwarf_expr import DWARFExprParser, DWARFExprOp
from elftools.dwarf.structs import DWARFStructs


class TestExprDumper(unittest.TestCase):
    structs32 = DWARFStructs(
            little_endian=True,
            dwarf_format=32,
            address_size=4)

    def setUp(self):
        self.visitor = ExprDumper(self.structs32)
        set_global_machine_arch('x64')

    def test_basic_single(self):
        self.assertEqual(self.visitor.dump_expr([0x1b]),
            'DW_OP_div')

        self.assertEqual(self.visitor.dump_expr([0x74, 0x82, 0x01]),
            'DW_OP_breg4 (rsi): 130')

        self.assertEqual(self.visitor.dump_expr([0x91, 0x82, 0x01]),
            'DW_OP_fbreg: 130')

        self.assertEqual(self.visitor.dump_expr([0x51]),
            'DW_OP_reg1 (rdx)')

        self.assertEqual(self.visitor.dump_expr([0x90, 16]),
            'DW_OP_regx: 16 (rip)')

        self.assertEqual(self.visitor.dump_expr([0x9d, 0x8f, 0x0A, 0x90, 0x01]),
            # Explaining the arguments is what the latest readelf does
            'DW_OP_bit_piece: size: 1295 offset: 144')

        self.assertEqual(self.visitor.dump_expr([0x0e, 0xff, 0x00, 0xff, 0x00, 0xff, 0x00, 0xff, 0x00]),
            'DW_OP_const8u: 71777214294589695')

    def test_basic_sequence(self):
        self.assertEqual(self.visitor.dump_expr([0x03, 0x01, 0x02, 0, 0, 0x06, 0x06]),
            'DW_OP_addr: 201; DW_OP_deref; DW_OP_deref')

        self.assertEqual(self.visitor.dump_expr([0x15, 0xFF, 0x0b, 0xf1, 0xff]),
            'DW_OP_pick: 255; DW_OP_const2s: -15')

        self.assertEqual(self.visitor.dump_expr([0x1d, 0x1e, 0x1d, 0x1e, 0x1d, 0x1e]),
            'DW_OP_mod; DW_OP_mul; DW_OP_mod; DW_OP_mul; DW_OP_mod; DW_OP_mul')

        # 0xe0 maps to both DW_OP_GNU_push_tls_address and DW_OP_lo_user, so
        # check for both to prevent non-determinism.
        self.assertIn(self.visitor.dump_expr([0x08, 0x0f, 0xe0]),
                      ('DW_OP_const1u: 15; DW_OP_GNU_push_tls_address',
                       'DW_OP_const1u: 15; DW_OP_lo_user'))


class TestParseExpr(unittest.TestCase):
    structs32 = DWARFStructs(
            little_endian=True,
            dwarf_format=32,
            address_size=4)

    def setUp(self):
        set_global_machine_arch('x64')

    def test_single(self):
        p = DWARFExprParser(self.structs32)
        lst = p.parse_expr([0x1b])
        self.assertEqual(lst, [DWARFExprOp(op=0x1B, op_name='DW_OP_div', args=[], offset=0)])

        lst = p.parse_expr([0x90, 16])
        self.assertEqual(lst, [DWARFExprOp(op=0x90, op_name='DW_OP_regx', args=[16], offset=0)])

        lst = p.parse_expr([0xe0])
        self.assertEqual(len(lst), 1)
        # 0xe0 maps to both DW_OP_GNU_push_tls_address and DW_OP_lo_user, so
        # check for both to prevent non-determinism.
        self.assertIn(lst[0], [
            DWARFExprOp(op=0xe0, op_name='DW_OP_GNU_push_tls_address', args=[], offset=0),
            DWARFExprOp(op=0xe0, op_name='DW_OP_lo_user', args=[], offset=0)])

        # Real life example:
        # push_object_address
        # deref
        # dup
        # bra 4
        # lit0
        # skip 3
        # lit4
        # minus
        # deref
        lst = p.parse_expr([0x97,0x6,0x12,0x28,0x4,0x0,0x30,0x2F,0x3,0x0,0x34,0x1C,0x6])
        self.assertEqual(len(lst), 9)
        self.assertEqual(lst, [
            DWARFExprOp(op=0x97, op_name='DW_OP_push_object_address', args=[], offset=0),
            DWARFExprOp(op=0x6,  op_name='DW_OP_deref', args=[], offset=1),
            DWARFExprOp(op=0x12, op_name='DW_OP_dup', args=[], offset=2),
            DWARFExprOp(op=0x28, op_name='DW_OP_bra', args=[4], offset=3),
            DWARFExprOp(op=0x30, op_name='DW_OP_lit0', args=[], offset=6),
            DWARFExprOp(op=0x2f, op_name='DW_OP_skip', args=[3], offset=7),
            DWARFExprOp(op=0x34, op_name='DW_OP_lit4', args=[], offset=10),
            DWARFExprOp(op=0x1c, op_name='DW_OP_minus', args=[], offset=11),
            DWARFExprOp(op=0x6,  op_name='DW_OP_deref', args=[], offset=12)])
        
        # This expression blob came from the test binary in issue 508,
        # DT_TAG_variable at 0x2a48C, DW_AT_location
        lst = p.parse_expr([0x5f, 0xf0])
        self.assertEqual(len(lst), 2)
        self.assertEqual(lst, [
            DWARFExprOp(op=0x5f, op_name='DW_OP_reg15', args=[], offset=0),
            DWARFExprOp(op=0xf0,  op_name='DW_OP_GNU_uninit', args=[], offset=1)])

if __name__ == '__main__':
    unittest.main()



# The test_gnudebugaltlink* and test_debugsup* files have been generated as
# follows:
# $ cat test_sup.c
# int main(int argc, char** argv)
# {
#        return argc;
# }
#
# $ gcc test_sup.c -o test_debugsup1
# $ gcc test_sup.c -o test_debugsup2
# $ dwz test_debugsup1 test_debugsup2 -m test_debugsup.common --dwarf-5
#
# $ gcc test_sup.c -o test_gnudebugaltlink1
# $ gcc test_sup.c -o test_gnudebugaltlink2
# $ dwz test_gnudebugaltlink1 test_gnudebugaltlink2 -m test_gnudebugaltlink.common

import unittest
import os

from elftools.elf.elffile import ELFFile

class TestDWARFSupplementaryObjects(unittest.TestCase):

    def test_gnudebugaltlink_no_followlinks(self):
        path = os.path.join('test', 'testfiles_for_unittests',
                            'test_gnudebugaltlink1.debug')
        with open(path, 'rb') as f:
            elffile = ELFFile(f)
            # Check that we don't have a supplementary_dwarfinfo
            dwarfinfo = elffile.get_dwarf_info(follow_links=False)
            self.assertIsNone(dwarfinfo.supplementary_dwarfinfo)
            # Check that imported units are present
            self.assertTrue(any(die.tag == 'DW_TAG_imported_unit'
                for cu in dwarfinfo.iter_CUs()
                for die in cu.iter_DIEs()))
            # Check that DW_FORM_GNU_strp_alt keep their raw_value.
            for cu in dwarfinfo.iter_CUs():
                for die in cu.iter_DIEs():
                    attrs = die.attributes
                    if ('DW_AT_name' in attrs and
                        attrs['DW_AT_name'].form == 'DW_FORM_GNU_strp_alt'):
                        self.assertEqual(attrs['DW_AT_name'].value,
                                         attrs['DW_AT_name'].raw_value)

    def test_gnudebugaltlink_followlinks(self):
        base_dir = os.path.join(b'test', b'testfiles_for_unittests')
        path = os.path.join(base_dir, b'test_gnudebugaltlink1.debug')
        with ELFFile.load_from_path(path) as elffile:
            # Check that we do have a supplementary_dwarfinfo
            dwarfinfo = elffile.get_dwarf_info()
            self.assertIsNotNone(dwarfinfo.supplementary_dwarfinfo)
            # Check that imported units are replaced by what they refer to.
            self.assertTrue(all(die.tag != 'DW_TAG_imported_unit'
                for cu in dwarfinfo.iter_CUs()
                for die in cu.iter_DIEs()))
            # Check that DW_FORM_GNU_strp_alt get a proper reference
            for cu in dwarfinfo.iter_CUs():
                for die in cu.iter_DIEs():
                    attrs = die.attributes
                    if ('DW_AT_name' in attrs and attrs['DW_AT_name'].form ==
                            'DW_FORM_GNU_strp_alt'):
                        self.assertIsInstance(attrs['DW_AT_name'].value, bytes)

    def test_debugsup_no_followlinks(self):
        path = os.path.join('test', 'testfiles_for_unittests',
                            'test_debugsup1.debug')
        with ELFFile.load_from_path(path) as elffile:
            # Check that we don't have a supplementary_dwarfinfo
            dwarfinfo = elffile.get_dwarf_info(follow_links=False)
            self.assertIsNone(dwarfinfo.supplementary_dwarfinfo)
            # Check that imported units are present
            self.assertTrue(any(die.tag == 'DW_TAG_imported_unit'
                for cu in dwarfinfo.iter_CUs()
                for die in cu.iter_DIEs()))
            # Check that DW_FORM_GNU_strp_alt keep their raw_value.
            for cu in dwarfinfo.iter_CUs():
                for die in cu.iter_DIEs():
                    attrs = die.attributes
                    if ('DW_AT_name' in attrs and
                        attrs['DW_AT_name'].form == 'DW_FORM_strp_sup'):
                        self.assertEqual(attrs['DW_AT_name'].value,
                                         attrs['DW_AT_name'].raw_value)

    def test_debugsup_followlinks(self):
        base_dir = os.path.join(b'test', b'testfiles_for_unittests')
        path = os.path.join(base_dir, b'test_debugsup1.debug')
        with ELFFile.load_from_path(path) as elffile:
            # Check that we do have a supplementary_dwarfinfo
            dwarfinfo = elffile.get_dwarf_info()
            self.assertIsNotNone(dwarfinfo.supplementary_dwarfinfo)
            # Check that imported units are replaced by what they refer to.
            self.assertTrue(all(die.tag != 'DW_TAG_imported_unit'
                for cu in dwarfinfo.iter_CUs()
                for die in cu.iter_DIEs()))
            # Check that DW_FORM_GNU_strp_alt get a proper reference
            for cu in dwarfinfo.iter_CUs():
                for die in cu.iter_DIEs():
                    attrs = die.attributes
                    if ('DW_AT_name' in attrs and attrs['DW_AT_name'].form ==
                            'DW_FORM_strp_sup'):
                        self.assertIsInstance(attrs['DW_AT_name'].value, bytes)



#-------------------------------------------------------------------------------
# elftools: dwarf/constants.py
#
# Constants and flags
#
# Eli Bendersky (eliben@gmail.com)
# This code is in the public domain
#-------------------------------------------------------------------------------

# Inline codes
#
DW_INL_not_inlined = 0
DW_INL_inlined = 1
DW_INL_declared_not_inlined = 2
DW_INL_declared_inlined = 3


# Source languages
#
DW_LANG_C89 = 0x0001
DW_LANG_C = 0x0002
DW_LANG_Ada83 = 0x0003
DW_LANG_C_plus_plus = 0x0004
DW_LANG_Cobol74 = 0x0005
DW_LANG_Cobol85 = 0x0006
DW_LANG_Fortran77 = 0x0007
DW_LANG_Fortran90 = 0x0008
DW_LANG_Pascal83 = 0x0009
DW_LANG_Modula2 = 0x000a
DW_LANG_Java = 0x000b
DW_LANG_C99 = 0x000c
DW_LANG_Ada95 = 0x000d
DW_LANG_Fortran95 = 0x000e
DW_LANG_PLI = 0x000f
DW_LANG_ObjC = 0x0010
DW_LANG_ObjC_plus_plus = 0x0011
DW_LANG_UPC = 0x0012
DW_LANG_D = 0x0013
DW_LANG_Python = 0x0014
DW_LANG_OpenCL = 0x0015
DW_LANG_Go = 0x0016
DW_LANG_Modula3 = 0x0017
DW_LANG_Haskell = 0x0018
DW_LANG_C_plus_plus_03 = 0x0019
DW_LANG_C_plus_plus_11 = 0x001a
DW_LANG_OCaml = 0x001b
DW_LANG_Rust = 0x001c
DW_LANG_C11 = 0x001d
DW_LANG_Swift = 0x001e
DW_LANG_Julia = 0x001f
DW_LANG_Dylan = 0x0020
DW_LANG_C_plus_plus_14 = 0x0021
DW_LANG_Fortran03 = 0x0022
DW_LANG_Fortran08 = 0x0023
DW_LANG_RenderScript = 0x0024
DW_LANG_BLISS = 0x0025
DW_LANG_Mips_Assembler = 0x8001
DW_LANG_Upc = 0x8765
DW_LANG_HP_Bliss = 0x8003
DW_LANG_HP_Basic91 = 0x8004
DW_LANG_HP_Pascal91 = 0x8005
DW_LANG_HP_IMacro = 0x8006
DW_LANG_HP_Assembler = 0x8007
DW_LANG_GOOGLE_RenderScript = 0x8e57
DW_LANG_BORLAND_Delphi = 0xb000


# Encoding
#
DW_ATE_void = 0x0
DW_ATE_address = 0x1
DW_ATE_boolean = 0x2
DW_ATE_complex_float = 0x3
DW_ATE_float = 0x4
DW_ATE_signed = 0x5
DW_ATE_signed_char = 0x6
DW_ATE_unsigned = 0x7
DW_ATE_unsigned_char = 0x8
DW_ATE_imaginary_float = 0x9
DW_ATE_packed_decimal = 0xa
DW_ATE_numeric_string = 0xb
DW_ATE_edited = 0xc
DW_ATE_signed_fixed = 0xd
DW_ATE_unsigned_fixed = 0xe
DW_ATE_decimal_float = 0xf
DW_ATE_UTF = 0x10
DW_ATE_UCS = 0x11
DW_ATE_ASCII = 0x12
DW_ATE_lo_user = 0x80
DW_ATE_hi_user = 0xff
DW_ATE_HP_float80 = 0x80
DW_ATE_HP_complex_float80 = 0x81
DW_ATE_HP_float128 = 0x82
DW_ATE_HP_complex_float128 = 0x83
DW_ATE_HP_floathpintel = 0x84
DW_ATE_HP_imaginary_float80 = 0x85
DW_ATE_HP_imaginary_float128 = 0x86


# Access
#
DW_ACCESS_public = 1
DW_ACCESS_protected = 2
DW_ACCESS_private = 3


# Visibility
#
DW_VIS_local = 1
DW_VIS_exported = 2
DW_VIS_qualified = 3


# Virtuality
#
DW_VIRTUALITY_none = 0
DW_VIRTUALITY_virtual = 1
DW_VIRTUALITY_pure_virtual = 2


# ID case
#
DW_ID_case_sensitive = 0
DW_ID_up_case = 1
DW_ID_down_case = 2
DW_ID_case_insensitive = 3


# Calling convention
#
DW_CC_normal = 0x1
DW_CC_program = 0x2
DW_CC_nocall = 0x3
DW_CC_pass_by_reference = 0x4
DW_CC_pass_by_valuee = 0x5


# Ordering
#
DW_ORD_row_major = 0
DW_ORD_col_major = 1


# Line program opcodes
#
DW_LNS_copy = 0x01
DW_LNS_advance_pc = 0x02
DW_LNS_advance_line = 0x03
DW_LNS_set_file = 0x04
DW_LNS_set_column = 0x05
DW_LNS_negate_stmt = 0x06
DW_LNS_set_basic_block = 0x07
DW_LNS_const_add_pc = 0x08
DW_LNS_fixed_advance_pc = 0x09
DW_LNS_set_prologue_end = 0x0a
DW_LNS_set_epilogue_begin = 0x0b
DW_LNS_set_isa = 0x0c
DW_LNE_end_sequence = 0x01
DW_LNE_set_address = 0x02
DW_LNE_define_file = 0x03
DW_LNE_set_discriminator = 0x04
DW_LNE_lo_user = 0x80
DW_LNE_hi_user = 0xff

# Line program header content types
#
DW_LNCT_path = 0x01
DW_LNCT_directory_index = 0x02
DW_LNCT_timestamp = 0x03
DW_LNCT_size = 0x04
DW_LNCT_MD5 = 0x05
DW_LNCT_lo_user = 0x2000
DW_LNCT_LLVM_source = 0x2001
DW_LNCT_LLVM_is_MD5 = 0x2002
DW_LNCT_hi_user = 0x3fff

# Call frame instructions
#
# Note that the first 3 instructions have the so-called "primary opcode"
# (as described in DWARFv3 7.23), so only their highest 2 bits take part
# in the opcode decoding. They are kept as constants with the low bits masked
# out, and the callframe module knows how to handle this.
# The other instructions use an "extended opcode" encoded just in the low 6
# bits, with the high 2 bits, so these constants are exactly as they would
# appear in an actual file.
#
DW_CFA_advance_loc = 0b01000000
DW_CFA_offset = 0b10000000
DW_CFA_restore = 0b11000000
DW_CFA_nop = 0x00
DW_CFA_set_loc = 0x01
DW_CFA_advance_loc1 = 0x02
DW_CFA_advance_loc2 = 0x03
DW_CFA_advance_loc4 = 0x04
DW_CFA_offset_extended = 0x05
DW_CFA_restore_extended = 0x06
DW_CFA_undefined = 0x07
DW_CFA_same_value = 0x08
DW_CFA_register = 0x09
DW_CFA_remember_state = 0x0a
DW_CFA_restore_state = 0x0b
DW_CFA_def_cfa = 0x0c
DW_CFA_def_cfa_register = 0x0d
DW_CFA_def_cfa_offset = 0x0e
DW_CFA_def_cfa_expression = 0x0f
DW_CFA_expression = 0x10
DW_CFA_offset_extended_sf = 0x11
DW_CFA_def_cfa_sf = 0x12
DW_CFA_def_cfa_offset_sf = 0x13
DW_CFA_val_offset = 0x14
DW_CFA_val_offset_sf = 0x15
DW_CFA_val_expression = 0x16
DW_CFA_GNU_window_save = 0x2d # Used on SPARC, not in the corpus
DW_CFA_AARCH64_negate_ra_state = 0x2d
DW_CFA_GNU_args_size = 0x2e


# Compilation unit types
#
# DWARFv5 introduces the "unit_type" field to each CU header, allowing
# individual CUs to indicate whether they're complete, partial, and so forth.
# See DWARFv5 3.1 ("Unit Entries") and 7.5.1 ("Unit Headers").
DW_UT_compile = 0x01
DW_UT_type = 0x02
DW_UT_partial = 0x03
DW_UT_skeleton = 0x04
DW_UT_split_compile = 0x05
DW_UT_split_type = 0x06
DW_UT_lo_user = 0x80
DW_UT_hi_user = 0xff



#!/usr/bin/env python
#-------------------------------------------------------------------------------
# test/run_examples_test.py
#
# Run the examples and compare their output to a reference
#
# Seva Alekseyev (sevaa@sprynet.com)
# This code is in the public domain
#
# This runs and times in-memory firehose DWARF parsing on all files from the dwarfdump autotest.
# The idea was to isolate the performance of the struct parsing logic alone.
#-------------------------------------------------------------------------------
from io import BytesIO
import os, sys, time, argparse

from utils import is_in_rootdir

sys.path.insert(0, '.')

from elftools.elf.elffile import ELFFile
from elftools.dwarf.dwarfinfo import DWARFInfo
from elftools.dwarf.locationlists import LocationParser

def parse_dwarf(ef, args):
    di = ef.get_dwarf_info()
    llp = LocationParser(di.location_lists())
    ranges = di.range_lists()
    for cu in di.iter_CUs():
        ver = cu.header.version
        if args.lineprog:
            # No way to isolate lineprog parsing :(
            di.line_program_for_CU(cu).get_entries()
        for die in cu.iter_DIEs():
            for (_, attr) in die.attributes.items():
                if args.locs and LocationParser.attribute_has_location(attr, ver):
                    llp.parse_from_attribute(attr, ver, die)
                elif args.ranges and attr.name == "DW_AT_ranges":
                    if ver >= 5:
                        ranges.get_range_list_at_offset_ex(attr.value)
                    else:
                        ranges.get_range_list_at_offset(attr.value)

def slurp(filename):
    with open(filename, "rb") as file:
        return BytesIO(file.read())

def main():
    if not is_in_rootdir():
        print('Error: Please run me from the root dir of pyelftools!', file=sys.stderr)
        return 1
    
    argparser = argparse.ArgumentParser()
    argparser.add_argument('-l', action='store_true', dest='locs')
    argparser.add_argument('-r', action='store_true', dest='ranges')
    argparser.add_argument('-p', action='store_true', dest='lineprog')
    args = argparser.parse_args()
    
    root = os.path.join('.', 'test', 'testfiles_for_dwarfdump')
    filenames = [filename for filename in os.listdir(root) if os.path.splitext(filename)[1] == '.elf']
    fileblobs = [slurp(os.path.join(root, filename)) for filename in filenames]
    start_time = time.time()
    for stream in fileblobs:
        parse_dwarf(ELFFile(stream), args)
    print("--- %s seconds ---" % (time.time() - start_time))


if __name__ == '__main__':
    sys.exit(main())

# To profile:
# python -m cProfile -s tottime test/run_parser_perf_test.py 


#------------------------------------------------------------------------------
# elftools tests
#
# Lukas Dresel (lukas.dresel@cs.ucsb.edu)
# This code is in the public domain
#------------------------------------------------------------------------------
import unittest
import os

from elftools.elf.elffile import ELFFile
from elftools.elf.segments import NoteSegment


class TestCoreNotes(unittest.TestCase):
    """ This test ensures that core dump specific notes
        are properly analyzed. Specifically, tests for a
        regression where MIPS PRPSINFO structures would be
        parsed incorrectly due to being treated as 16-bit
        fields instead of 32-bit fields.
    """
    @classmethod
    def setUpClass(cls):
       cls._core_file = open(os.path.join('test',
                             'testfiles_for_unittests', 'core_linux32_qemu_mips.elf'),
                             'rb')

    def test_core_prpsinfo(self):
        elf = ELFFile(self._core_file)
        for segment in elf.iter_segments():
            if not isinstance(segment, NoteSegment):
                continue
            for note in segment.iter_notes():
                if note['n_type'] != 'NT_PRPSINFO':
                    continue
                desc = note['n_desc']
                self.assertEqual(desc['pr_state'], 0)
                self.assertEqual(desc['pr_sname'], b'\0')
                self.assertEqual(desc['pr_zomb'], 0)
                self.assertEqual(desc['pr_nice'], 0)
                self.assertEqual(desc['pr_flag'], 0x0)
                self.assertEqual(desc['pr_uid'], 1000)
                self.assertEqual(desc['pr_gid'], 1000)
                self.assertEqual(desc['pr_pid'], 449015)
                self.assertEqual(desc['pr_ppid'], 4238)
                self.assertEqual(desc['pr_pgrp'], 449015)
                self.assertEqual(desc['pr_sid'], 4238)
                self.assertEqual(
                    desc['pr_fname'],
                    b'crash'.ljust(16, b'\0'))
                self.assertEqual(
                    desc['pr_psargs'],
                    b'./crash '.ljust(80, b'\x00'))

    @classmethod
    def tearDownClass(cls):
        cls._core_file.close()


if __name__ == '__main__':
    unittest.main()
